import { ref, reactive, watch, h, defineComponent, onMounted } from 'vue';
import { useForm as useForm$1 } from '@inertiajs/inertia-vue3';
import Draggable from 'vuedraggable';

const defaultTransformFilters = filters => {
    for (let key in filters) {
        filters[key] = filters[key].value;
    }
    return filters;
};
const useIndex = function useIndex({ route, syncUrl = false, defaultPerPage = 10, filters = {}, sortBy = {}, transformFilters = defaultTransformFilters }) {
    const search = ref("");
    let index = reactive({
        processing: false,
        perPage: defaultPerPage,
        items: [],
        hasNextPage: false,
        hasPrevPage: false,
        currentPage: 1,
        search: search,
        fromItem: 0,
        toItem: 0,
        totalItems: 0,
        __route: route,
        __hooks: {
            beforeUpdate: [],
            afterUpdate: [],
        },
        filters: reactive(filters),
        sortBy: reactive(sortBy),
        loadItems() {
            this.processing = true;
            for (let i in this.__hooks.beforeUpdate) {
                this.__hooks.beforeUpdate[i](this);
            }
            fetch(this.__getUrl())
                .then(response => response.json())
                .then((data) => this.__update(data))
                .catch(() => this.processing = false);
        },
        reloadOnChange(value) {
            watch(value, () => this.reload());
        },
        reload() {
            return this.loadItems();
        },
        addFilter(filter) {
            this.filters.push(filter);
            this.reload();
        },
        removeFilter(filter) {
            //
            this.reload();
        },
        getLastPage() {
            return Math.ceil(this.totalItems / this.perPage);
        },
        setPage(newPage) {
            if (this.newPage < 1 || this.newPage > this.getLastPage()) {
                return;
            }
            this.currentPage = newPage;
            this.reload();
        },
        nextPage() {
            this.setPage(this.currentPage + 1);
        },
        prevPage() {
            this.setPage(this.currentPage - 1);
        },
        lastPage() {
            this.setPage(this.getLastPage());
        },
        updateSearch(e) {
            console.log({ e });
            if (e instanceof Event) {
                this.search = e.target.value;
            }
            else {
                this.search = e;
            }
            this.setPage(1);
        },
        isSortedBy(key, direction = null) {
            if (direction == 'desc' || direction == 'asc') {
                return this.sortBy.includes(`${key}.${direction}`);
            }
            return this.sortBy.includes(`${key}.desc`) || this.sortBy.includes(`${key}.asc`);
        },
        addSortBy(key, direction = 'asc') {
            let value = `${key}.${direction}`;
            if (this.isSortedBy(key, direction == 'asc' ? 'decs' : 'asc')) {
                this.removeSortBy(key);
            }
            if (!this.sortBy.includes(value)) {
                this.sortBy.push(value);
            }
        },
        removeSortBy(key) {
            let index = this.sortBy.indexOf(`${key}.desc`);
            if (index > -1) {
                this.sortBy.splice(index);
            }
            index = this.sortBy.indexOf(`${key}.asc`);
            if (index > -1) {
                this.sortBy.splice(index);
            }
        },
        beforeUpdate(cb) {
            this.__hooks.beforeUpdate.push(cb);
        },
        onUpdate(cb) {
            this.__hooks.afterUpdate.push(cb);
        },
        __update(data) {
            this.items = data.data;
            this.totalItems = data.meta.total;
            this.fromItem = data.meta.from;
            this.toItem = data.meta.to;
            this.perPage = data.meta.per_page;
            this.hasNextPage = this.currentPage < this.getLastPage();
            this.hasPrevPage = this.currentPage > 1;
            this.processing = false;
            this.__syncUrl();
            for (let i in this.__hooks.afterUpdate) {
                this.__hooks.afterUpdate[i](this);
            }
        },
        __syncUrl() {
            if (!syncUrl) {
                return;
            }
            let prefix = '';
            if (typeof syncUrl === 'string') {
                prefix = `${syncUrl}`;
            }
            let params = {};
            let p = this.__getParams();
            for (let key in p) {
                params[prefix + key] = p[key];
            }
        },
        __getUrl() {
            const searchParams = new URLSearchParams([]);
            let params = this.__getParams();
            for (let key in params) {
                if (Array.isArray(params[key])) {
                    for (let i = 0; i < params[key].length; i++) {
                        searchParams.append(`${key}[]`, params[key][i]);
                    }
                }
                else {
                    searchParams.append(key, encodeURIComponent(params[key]));
                }
            }
            for (let i = 0; i < this.sortBy.length; i++) {
                searchParams.append('sortBy[]', this.sortBy[i]);
            }
            return `${this.__route}?${searchParams.toString()}`;
        },
        __getParams() {
            let params = {
                page: this.currentPage,
                search: this.search,
            };
            let filters = JSON.parse(JSON.stringify(this.filters));
            if (transformFilters) {
                filters = transformFilters(filters);
            }
            for (let key in filters) {
                if (!filters[key]) {
                    continue;
                }
                params[`filter.${key}`] = filters[key];
            }
            return params;
        }
    });
    index.reloadOnChange(search);
    return index;
};

const useForm = function ({ route, data, method = 'put', onClean = () => { }, onDirty = () => { }, onCancelToken = () => { }, onBefore = () => { }, onStart = () => { }, onProgress = () => { }, onFinish = () => { }, onCancel = () => { }, onSuccess = () => { }, onError = () => { }, } = {}) {
    const inertiaForm = useForm$1(data);
    let defaults = JSON.stringify(data);
    let form = reactive({
        ...inertiaForm,
        submit(e) {
            if (e instanceof Event) {
                e.preventDefault();
            }
            this.__submit(method, route, {
                headers: { Accept: 'application/json' },
                onCancelToken,
                onBefore,
                onStart,
                onProgress,
                onFinish,
                onCancel,
                onSuccess,
                onError
            });
            defaults = JSON.stringify(this.data());
        },
        get: undefined,
        post: undefined,
        put: undefined,
        patch: undefined,
        delete: undefined,
        __submit: inertiaForm.submit
    });
    watch(form, () => {
        form.isDirty = JSON.stringify(form.data()) != defaults;
        if (form.isDirty) {
            onDirty(form);
        }
        else {
            onClean(form);
        }
    }, { immediate: true, deep: true });
    return form;
};

const uuid$2 = require('uuid').v4;
function parseOrderRecursive(list) {
    let order = [];
    for (let i = 0; i < list.items.length; i++) {
        order.push({
            id: list.items[i].value.id,
            children: parseOrderRecursive(list.items[i].children),
        });
    }
    return order;
}
const useTree = (items = [], options = {}) => {
    const list = reactive({
        items: [],
        onOrderChange: options.onOrderChange ? options.onOrderChange : (order) => { },
        push(item, children = []) {
            this.items.push({
                children: useTree(children),
                value: item,
            });
        },
        pop() {
            return this.items.pop();
        },
        setItems(list) {
            let items = [];
            for (let i = 0; i < list.length; i++) {
                items.push({
                    children: useTree(list[i].children),
                    uuid: uuid$2(),
                    value: list[i].value
                });
            }
            this.items = items;
        },
        updateOnChange(items) {
            watch(() => items, () => list.setItems(items), { immediate: true, deep: true });
        },
        getOrder() {
            return parseOrderRecursive(this);
        }
    });
    list.setItems(items);
    const originalOrder = useOriginal(list.getOrder);
    watch(list, () => {
        const order = list.getOrder();
        if (originalOrder.matches(order)) {
            return;
        }
        originalOrder.update(order);
        list.onOrderChange(order);
    }, { immediate: true, deep: true });
    return list;
};

const useSaveQueue = function () {
    const saveQueue = reactive({
        jobs: {},
        isDirty: false,
        add(key, job, callback = () => null) {
            this.jobs[key] = { job, callback };
        },
        remove(key) {
            delete this.jobs[key];
        },
        async save() {
            let promises = [];
            for (let key in this.jobs) {
                let func = async () => {
                    await this.jobs[key].job();
                    this.jobs[key].callback();
                };
                promises.push(func());
            }
            this.jobs = {};
            await Promise.all(promises.map(p => p.catch(e => e)));
        },
    });
    watch(saveQueue, () => {
        saveQueue.isDirty = Object.keys(saveQueue.jobs).length > 0;
    }, { intermediate: true });
    return saveQueue;
};

const useOriginal = (value) => {
    const original = reactive({
        raw: value,
        stringified: JSON.stringify(value),
        update(value) {
            this.raw = value;
            this.stringified = JSON.stringify(value);
        },
        matches(value) {
            return JSON.stringify(value) == this.stringified;
        },
    });
    return original;
};

const translatable = (values, locale) => {
    const translatable = reactive({
        value: values[locale.value] || null,
        values,
        locale,
        setLocale(locale) {
            this.locale = locale;
        },
    });
    watch(() => translatable.locale, (locale) => {
        translatable.value = translatable.values[locale];
    }, { intermediate: true });
    return translatable;
};

const debounce = require('lodash.debounce');
const Input = function ({ modelModifiers = {} }, { attrs = {}, emit }) {
    let onInput = ({ target }) => {
        emit('update:modelValue', target.value);
    };
    if (modelModifiers.debounce) {
        onInput = debounce(onInput, 200);
    }
    return h(`input`, {
        ...attrs,
        value: attrs.modelValue,
        onInput,
    });
};

const uuid$1 = require('uuid').v4;
const template$1 = `
    <Draggable
        v-if="value"
        v-model="value"
        v-bind="dragOptions"
        :group="group"
        itemKey="uuid"
        tag="transition-group"
        :component-data="{
            name: !drag ? 'flip-list' : null,
            wrap: true,
        }"
        @start="drag = true"
        @end="drag = false"
    >
        <template #item="{ element }">
            <div>
                <component
                    :is="sections[getSectionKey(element.component)]"
                    :modelValue="element.value"
                    @update:modelValue="(e) => updateElement(element, e)"
                />
            </div>
        </template>
    </Draggable>
`;
const Sections = defineComponent({
    template: template$1,
    components: { Draggable },
    emits: ['update:modelValue'],
    props: {
        modelValue: {
            type: Array,
            required: true,
        },
        sections: {
            type: Object,
            required: true,
        },
        group: {
            type: String,
            default: 'sections',
        },
        dragOptions: {
            type: Object,
            default: () => ({
                animation: 200,
                ghostClass: 'ghost',
            }),
        },
    },
    setup(props, { emit }) {
        const drag = ref(false);
        const updateElement = (element, value) => {
            element.value = value;
        };
        const getSectionKey = (section) => {
            const json = JSON.stringify(section);
            for (let key in props.sections) {
                if (JSON.stringify(props.sections[key]) == json) {
                    return key;
                }
            }
            return '';
        };
        const parseValue = (value) => {
            let b = [];
            for (let i in value) {
                b.push({
                    uuid: uuid$1(),
                    key: value[i].type,
                    value: value[i].value,
                    component: props.sections[value[i].type],
                });
            }
            return b;
        };
        const transformValue = (value) => {
            let b = [];
            for (let i in value) {
                b.push({
                    type: getSectionKey(value[i].component),
                    value: value[i].value || {},
                });
            }
            return b;
        };
        const value = ref(parseValue(props.modelValue));
        watch(() => value, () => emit('update:modelValue', transformValue(value.value)), { deep: true });
        return { drag, updateElement, getSectionKey, value };
    }
});

const uuid = require('uuid').v4;
const template = `
    <Draggable
        v-model="drawers"
        :group="{
            name: group,
            pull: 'clone',
            put: false,
        }"
        :sort="false"
        :clone="cloneSection"
    >
        <template #item="{ element }">
            <div>
                <component :is="element" />
            </div>
        </template>
    </Draggable>
`;
const Cabinet = defineComponent({
    template,
    components: { Draggable },
    props: { group: { type: String, default: 'sections' } },
    setup({}, { slots }) {
        const drawers = ref();
        onMounted(() => {
            if (slots.default) {
                drawers.value = slots.default();
            }
        });
        const cloneSection = (el) => ({
            uuid: uuid(),
            component: el.props.draws,
        });
        return { cloneSection, drawers };
    }
});

export { Cabinet, Input, Sections, translatable, useForm, useIndex, useOriginal, useSaveQueue, useTree };
